=pod

=head1 NAME

WWW::NYTimes::Parser

=head1 SUMMARY 

TBW

=head1 DESCRIPTION

"All materials contained on this site are protected by United States copyright law and may not be reproduced, distributed, transmitted, displayed, published or broadcast without the prior written permission of The New York Times Company. You may not alter or remove any trademark, copyright or other notice from copies of the content. 

"However, you may download material from The New York Times on the Web (one machine readable copy and one print copy per page) for your personal, noncommercial use only."

http://www.nytimes.com/info/help/copyright.html

"2.3 You may download or copy the Content and other downloadable items displayed on the Service for personal use only, provided that you maintain all copyright and other notices contained therein. Copying or storing of any Content for other than personal use is expressly prohibited without prior written permission from The New York Times Rights and Permissions Department, or the copyright holder identified in the copyright notice contained in the Content."

http://www.nytimes.com/info/help/agree.html

=cut

package WWW::NYTimes::Parser;
use strict;

use constant NYTIMES                => "http://www.nytimes.com";
use constant INDEX                  => "/info/help/textpath.html";
use constant TEXTONLY               => "/pages/world/text";
use constant DEFAULT_SECTION_MATCH  => "^(\/pages\/)([a-z\/]+)(\/text)(.*)";
use constant DEFAULT_HEADLINE_MATCH => "^\/((\d\d\d\d)\/(\d\d)\/(\d\d))(.*)";

use vars qw ( @ISA @EXPORT $VERSION $DocBook );

@ISA = qw ( Exporter );
@EXPORT = qw (&parse_article);

$VERSION = 1.0;
$DocBook = 0;

use WWW::News::Utils;

use Data::Dumper;
use Exporter;

use HTML::LinkExtor;
use HTML::Parser;

use HTTP::Cookies;
use HTTP::Request;

use LWP::UserAgent;
use URI;

use constant PRINTED => "pagewanted=print";
use constant NOLOGIN => "Sign In or Register at NYTimes.com";

# Global objects 
my $agent   = undef;
my $cookies = undef;
my $parser  = undef;

# Globals variables
my $okay    = undef;
my $nologin = undef;
my $lasttag = undef;
my $article = undef;

my $articletitle = "";

my @sections  = ();
my @headlines = ();

my $sections_match  = undef;
my $headlines_match = undef;

my $gurl = undef;

=pod

=head1 PUBLIC METHODS

=cut

=pod

=head2 get_sections($pattern)

Returns an array of URLs

=cut

sub get_sections {
    my $pattern = shift;

    &_set_sections_match($pattern);
    @sections = ();

    my $ua    = LWP::UserAgent->new();

    my $p     = HTML::LinkExtor->new(\&_parse_sections) || die $!;
    my $res   = $ua->request(HTTP::Request->new(GET => NYTIMES.INDEX), sub {$p->parse($_[0])});

    return @sections;
}

sub _parse_sections {
    my($tag, %attr) = @_;
    return if $tag ne 'a';
    my $url = $attr{'href'};
    return unless ($url =~ /$sections_match/);
    push(@sections, $url);
}

sub _set_sections_match {
    my $pattern = shift;
    $sections_match = ($pattern) ? $pattern : DEFAULT_SECTION_MATCH; 
}

=pod

=head2 &get_headlines($url,$pattern)

Returns an array of URLs

=cut

sub get_headlines {
    my $url     = shift;
    my $pattern = shift;
    my $login   = shift;

    &_set_headlines_match($pattern);
    @headlines  = ();

    my $ua   = LWP::UserAgent->new();

    if ($login) {
	# HTTP::Cookies object or die
	$cookies = HTTP::Cookies::Netscape->new(
						File     => $login,
						AutoSave => 1,
						);
	
	# Read in the cookie file 
	$ua->cookie_jar($cookies);
    }

    my $p    = HTML::LinkExtor->new(\&_parse_headlines);
    my $res  = $ua->request(HTTP::Request->new(GET => $url), sub {$p->parse($_[0])});

    return @headlines;
}

sub _parse_headlines {
    my($tag, %attr) = @_;
    return if $tag ne 'a';
    my $url = $attr{'href'};
    return unless ($url =~ /$headlines_match/);
    push(@headlines, $url) unless (grep /^($url)$/,@headlines);
}

sub _set_headlines_match {
    my $pattern = shift;
    $headlines_match = ($pattern) ? $pattern : DEFAULT_HEADLINE_MATCH; 
    #print "Matching '$headlines_match'\n";
}

=pod

=head2 parse_article($url,$login)

Fetches the "printable" version of $url and returns the body of the article, stripped of ads and menus and all that other stuff. The resultant string is also stripped of all HTML.

If there is an error fetching the URI, the method returns a fault message containing the HTTP error number and message.

Returns a string.

=cut

sub parse_article {
    my $url   = shift;
    my $login = shift;

    $gurl = $url;

    $lasttag      = undef;
    $article      = undef;
    $articletitle = undef;
    $nologin      = 0;
    $okay         = 0;

    # Check to make sure there is a URL

    if (! $url) {
	return (undef,"No URL");
    }

    # Check to make sure it is a N.Y. Times URL

    my $uri  = URI->new($url) || return (undef,$!);
    my $host = $uri->host()   || return (undef,"Unable to determine host.");
    
    unless ($host =~ /(nytimes\.com)$/) {
	return (undef,"Unable to verify nytimes-ness of the URL.");
    }

    # Rather than checking to see if we have 
    # the correct query string for a "printable"
    # version, we just blast the query string
    # and tack on the correct CGI parameters.

    $url = (split(/\?/,$url))[0];
    $url = join("\?",$url,PRINTED);

    # HTML::Parser object or die
    $parser ||= HTML::Parser->new(
				  start_h => [ \&_start,"tagname, attr" ],
				  end_h   => [ \&_end,"tagname"   ],
				  text_h  => [ \&_text,"text"     ],
				  ) || return (undef,$!);

    
    if ((! $login) || (! -f $login)) {
	return (undef,"Failed to locate cookie file.");
    }
 
    # HTTP::Cookies object or die
    $cookies ||= HTTP::Cookies::Netscape->new(
					      File     => $login,
					      AutoSave => 1,
					      ) || return (undef,$!);
   
    # LWP::UserAgent object or die
    $agent ||= LWP::UserAgent->new() || return (undef,$!);

    # Read in the cookie file 
    $agent->cookie_jar($cookies);

    # Fetch the page
    my $res = $agent->request(HTTP::Request->new(GET => $url));
    
    # Make sure we got the page or die
    if ($res->{"_rc"} ne "200" ) {
	return (undef,"$res->{'_rc'} : $res->{'_msg'}");
    }

    # Munge munge munge
    $parser->parse($res->{"_content"});
    
    # Did we actually munge anything useful?
    if ($nologin) {
	return (undef,"Login failed.");
    }

    # Return the article

    if ($DocBook) { 
	$article = 
		"<part>".
		"<partinfo>".    
		"<title>$articletitle</title>".
                "</partinfo>".
		"<article>".
		#"<para><ulink url = '$url'>$url</ulink></para>".
		$article.
		"</article>".
		"</part>";

	&_clean_article(\$article);
	# &_add_releaseinfo(\$article,$url);
    }

    return ($article);
}

sub _clean_article {
    my $sr_text = shift;
    $$sr_text =~ s/<\/articleinfo>/<\/articleinfo><para>/;
    $$sr_text =~ s/<\/article>/<\/para><\/article>/;    
    $$sr_text =~ s/<para \/>/<\/para><para>/g;

#    $$_text =~ 
#	#
#	s/<\/articleinfo>(.*)<para \/>(.*)<\/article>
#	    #
#	    /<\/articleinfo><para>(.*)<\/para><para>(.*)<\/para><\/article>/gm;

}

sub _add_releaseinfo {
    my $sr_text = shift;
    my $url     = shift;
    print "URL is $url\n";
    $sr_text =~ s/(<ulink url = ")(" \/>)/$1$url$2/gm;
}

sub _start {
    my $tag   = shift;
    my $attrs = shift;

    if ($tag eq "nyt_text")         { $okay = 1; }
    if ($tag eq "nyt_links_onsite") { $okay = 0; }

    $lasttag = $tag || "";

    if (($okay) && ($tag eq "img") && ($attrs->{'src'} =~ /\/dropcap\/\w\.gif$/) ) {
	$article .= $attrs->{'alt'};
    }

    return unless ($DocBook);

    if (($tag eq "br") && ($okay)) {
	$article .= "<para />";
    }

    if ($tag eq "head") {
	$article .= "<articleinfo>";
    }

    if (($tag eq "meta") && ($attrs->{'name'} eq "NYT_HEADLINE")) { 
	my $title = $attrs->{'content'} || "";
	&WWW::News::Utils::encode(\$title);

	$articletitle = $title;
	$article .= "<title>$title</title>";
    }

    if (($tag eq "meta") && ($attrs->{'name'} eq "BY_LINE")) { 
	$attrs->{'content'} =~ /^(By)(\s)*(\w+)(\s)*(.*)$/i;
	
	my $firstname = $3 || "";
	my $surname   = $5 || "";

	&WWW::News::Utils::encode(\$firstname);
	&WWW::News::Utils::encode(\$surname);

	$article .= "<author><surname>$surname</surname><firstname>$firstname</firstname></author>";
    }

    if (($tag eq "meta") && ($attrs->{'name'} eq "FIRSTPAR")) { 
	my $abstract = $attrs->{'content'} || "";
	&WWW::News::Utils::encode(\$abstract);
	$article .= "<abstract><para>$abstract</para></abstract>";
    }

    if (($tag eq "meta") && ($attrs->{'name'} eq "pdate")) { 
	my $pubdate = $attrs->{'content'} || "";
	&WWW::News::Utils::encode(\$pubdate);
	$article .= "<pubdate>$pubdate</pubdate>";
    }

}

sub _end { 
    my $tag = shift;

    if ($tag eq "nyt_links_onsite") { $okay = 1; }
    if ($tag eq "nyt_text")         { $okay = 0; };

    return unless ($DocBook);

    if ($tag eq "head") {
	$article .= 
	    "<releaseinfo>".
	    "<ulink url = \"$gurl\" />".
	    "</releaseinfo>".
	    "<copyright><year>".
	    (&WWW::News::Utils::today())[0].
	    "</year><holder>New York Times Corporation</holder></copyright>".
	    "</articleinfo>";
    }

    if (($tag eq "p") && ($okay)) {
	$article .= "<para />";
    }
}

sub _text {
    my $text = shift || "";
    chomp $text;

    if (($lasttag eq "title") && ($text eq NOLOGIN)) {
	$nologin = 1;
    }

    return unless ($text && $okay);

    if ($DocBook) { &WWW::News::Utils::encode(\$text); }
    $article .= $text;
}

return 1;

=pod

=head1 TO DO 

=over 4

=item 

Allow for numeric quads when we check for nytimes.com URL.

=item 

Add hooks to return as DocBook XML as well as raw text.

=back

=head1 VERSION

1.0

=head1 AUTHOR

Aaron Straup Cope

=head1 SEE ALSO

L<SOAP::Lite>

L<HTML::Parser>

L<HTTP::Cookies>

L<HTTP::Request>

L<LWP::UserAgent>

L<URI>

=head1 LICENSE

Copyright (c) 2001-2002 Aaron Straup Cope. All Right Reserved.

This is free software, you may use it and distribute it under the same terms as Perl itself.

=cut

